/*
Vocero
Widget to combine news from various RSS feeds and read them out loud.
Aug 2005
Daniel Gobera
http://www.mexicochannel.net/gobera/vocero/
*/

// Global variables
var version="1.1"; // widget version
var feeds; // array of feeds available (rss bookmarks)
var newsMaster; // array that holds all the news read
var newsDisplay; // array that holds only the news that will be shown
var currentFeed; // pointer to the next feed to read
var req; // xmlhttprequest object
var reqCheckVersion; // xmlhttprequest object to check for new versions
var maxLength=500; // max length of an item's description
var selectedItem=-1; // index of the item currently selected
var needsRefresh=false; // if true, it must refresh when flippint to front side
var firstRun; // indicates if it is the widgets first run
var currentlyPlaying=false; // indicates if it is currently reading news
var currentlyBeingSpoken=null; // what is being spoken
var bookmarksLimit=40; // max number of bookmarks that can be handled
var bookmarksFound; // total number of bookmarks in the system
var homeURL="http://www.mexicochannel.net/gobera/vocero/"; // home URL of the widget
var checkVersionURL=homeURL+"?checkVersion=1"; // URL to check for new versions
var newVersion=0; // number of the latest version available to download

// Default preferences
var prefs=new Array();
prefs['refreshTime']=30; // the default time in minutes to refresh
prefs['headerVoice']="Victoria"; // the default voice to read the headers
prefs['textVoice']=""; // the default voice to read the text

// Names of the months
var months=new Array();
months[1]="Jan"; months[2]="Feb"; months[3]="Mar"; months[4]="Apr";
months[5]="May"; months[6]="Jun"; months[7]="Jul"; months[8]="Aug";
months[9]="Sep"; months[10]="Oct"; months[11]="Nov"; months[12]="Dec";

// Available system voices
var voices = new Array("--None--", "Agnes", "Albert", "Bad News", "Bahh", "Bells", 
	"Boing", "Bruce", "Bubbles", "Cellos", "Deranged", "Fred", "Good News", 
	"Hysterical", "Junior", "Kathy", "Pipe Organ", "Princess", "Ralph", 
	"Trinoids", "Vicki", "Victoria", "Whisper", "Zarvox");


// Setup the widget
function setup(){
	// assign a function for when it is removed
	if(window.widget) widget.onremove=widgetRemoved;
	// check if it is the first run
	if(getPreference("noFirstRun")=="1") firstRun=false;
	else{
		firstRun=true;
		// indicate it has run before
		setPreference("noFirstRun", "1");
	}
	// create done button
	var doneButton=document.getElementById("doneButton");
	createGenericButton(doneButton, "Done", hideBack);
	// display version
	document.getElementById("version").innerHTML=version;
	// show a status message
	showMessage("Loading feeds...");
	// read user preferences
	readPreferences();
	// refresh list of voices
	refreshVoices();
	// refresh preferences pane
	refreshPreferences(true);
	// start the engine
	waitAndRefresh(0);
}

//-----------------------------------------------------//
// Deal With Timing and Dispatching                    //
//-----------------------------------------------------//

// Wait secs time and refresh the whole thing
function waitAndRefresh(min){
	// wait the specified time and read the first feed
	setTimeout("refreshFeeds();", min*60000); // convert to miliseconds
}


// Initialize variables and start reading feeds
function refreshFeeds(){
	// check for new version
	checkNewVersion();
	// pause the speaking
	pauseSpeakItems();
	// clear search box
	document.getElementById("searchBox").value="";
	// initialize variables
	newsMaster=new Array();
	currentFeed=0;
	// start loading feeds
	loadNextFeed();
}


//-----------------------------------------------------//
// Deal With RSS Downloading and Parsing               //
//-----------------------------------------------------//

// Process a received RSS feed
function processResponseRSS(response){
	var f, tmpValue, cutPoint;
	var allItems=new Array();
	var item=new Array();
	var tags = new Array();
	// if there is a response, process the xml document
	if(!isNull(response)){
		// determine the tags names based on the feed type
		if(feeds[currentFeed]['type']=="rss1"){ // rss ver. 1.0
			tags['item'] = "item";
			tags['title'] = "title";
			tags['description'] = "description";
			tags['link'] = "link";
			tags['date'] = "date";
			tags['date2'] = "pubDate";
		}
		else if(feeds[currentFeed]['type']=="rss2"){ // rss ver. 2.0
			tags['item'] = "item";
			tags['title'] = "title";
			tags['description'] = "description";
			tags['link'] = "link";
			tags['date'] = "date";
			tags['date2'] = "pubDate";
		}
		// get the items in an array
		allItems = response.getElementsByTagName(tags['item']);
		// if it contains feeds, process them
		if(!isNull(allItems)){
			for(f=0;f<allItems.length;f++){
				// read the data into item array
				item=new Array();
				// source
				item['source'] = feeds[currentFeed]['name'];
				// title (try first without namespace, then with the specified one)
				tmpValue=allItems[f].getElementsByTagName(tags['title'])[0];
				if(isNull(tmpValue)) tmpValue=allItems[f].getElementsByTagNameNS(allItems[f].namespaceURI, tags['title'])[0];
				if(!isNull(tmpValue) && !isNull(tmpValue.firstChild))
					item['title']=tmpValue.firstChild.data;
				// description (try first without namespace, then with the specified one)
				tmpValue=allItems[f].getElementsByTagName(tags['description'])[0];
				if(isNull(tmpValue)) tmpValue=allItems[f].getElementsByTagNameNS(allItems[f].namespaceURI, tags['description'])[0];
				if(!isNull(tmpValue) && !isNull(tmpValue.firstChild)){
					item['description']=tmpValue.firstChild.data;
					// if it contains html tags
					if(item['description'].indexOf("<")>-1){
						// remove links
						item['description']=item['description'].replace(/<\/?a\s+.*?>/ig, "");
					}
					// if not, trim it to the max length
					else if(item['description'].length>maxLength){
						cutPoint=item['description'].substr(0, maxLength).lastIndexOf(" ");
						item['description']=item['description'].substr(0, cutPoint)+"...";
					}
				}
				// date (try first without namespace, then with the specified one)
				tmpValue=allItems[f].getElementsByTagName(tags['date'])[0];
				if(isNull(tmpValue)) tmpValue=allItems[f].getElementsByTagName(tags['date2'])[0];
				if(isNull(tmpValue)) tmpValue=allItems[f].getElementsByTagNameNS(allItems[f].namespaceURI, tags['date'])[0];
				if(isNull(tmpValue)) tmpValue=allItems[f].getElementsByTagNameNS("*", "date")[0];
				if(!isNull(tmpValue) && !isNull(tmpValue.firstChild))
					item['date']=tmpValue.firstChild.data;
				// parse the date
				if(!isNull(item['date']) && item['date'].length>5){
					item['date']=parseDate(item['date']);
				}
				// if it is null or not valid, use today's
				if(isNull(item['date']) || !isNumber(item['date'].getDate()))
					item['date']=new Date();
				// link (try first without namespace, then with the specified one)
				tmpValue=allItems[f].getElementsByTagName(tags['link'])[0];
				if(isNull(tmpValue)) tmpValue=allItems[f].getElementsByTagNameNS(allItems[f].namespaceURI, tags['link'])[0];
				if(!isNull(tmpValue) && !isNull(tmpValue.firstChild))
					item['link']=tmpValue.firstChild.data;
				// store the item in the master news array
				newsMaster.push(item);
			}
		}
	}
	else{ // if allItems is null, it was a failure
	}
	// increase the feed index to read
	currentFeed++;
	// if it was not the last one, read the next one
	if(currentFeed<feeds.length) loadNextFeed();
	// if it was the last one
	else{
		// sort the news by date descendently
		newsMaster.sort(compareDate);
		// copy the newsMaster array
		newsDisplay=newsMaster;
		// update the display
		updateDisplayedNews();
		// restart the engine
		waitAndRefresh(prefs['refreshTime']);
	}
}


// Load an XML RSS feed asynchronously
function loadNextFeed() {
	// if it is selected
	if(!isNull(feeds[currentFeed]) && isFeedSelected(feeds[currentFeed]['url'])){
		// display a message
		showMessage("Loading: "+feeds[currentFeed]['name']);
		// determine url and parameters
		req = new XMLHttpRequest();
		var url=feeds[currentFeed]['url'];
		req.setRequestHeader('If-Modified-Since', 'Wed, 15 Nov 1995 00:00:00 GMT');//Safari cache workaround
		req.onreadystatechange = processReqChange;
		// add a random parameter to the url to eliminate cache issues
		if(url.indexOf("?")!=-1) url += "&"+Math.random();
		else url += "?"+Math.random();
		// send the request to the server
		req.open("GET", url, true);
		req.send(null);
	}
	else{ // not selected
		processResponseRSS(null); // go to the next feed
	}
}

// Process the server's response
function processReqChange(){
	// verify that the status is "COMPLETE"
	if (req.readyState == 4) {
		// verify that the status response is "OK"
		if (req.status>=200 && req.status<300 && !isNull(req.responseXML)) { // if status is 2xx
			// create XML object
			response = req.responseXML.documentElement;
			// determine the type of the feed (rss1, rss2)
			if(req.responseXML.firstChild.tagName=="rss"){ // If it is an RSS 2.0 document
				// use RSS 2.0 format for RSS 0.9, 0.91, 0.92, 0.93, 0.94 and 2.0
				feeds[currentFeed]['type']="rss2"; // 
			}
			else if(req.responseXML.firstChild.tagName=="rdf:RDF"){ // If it is an RDF (RSS 1.0) document
				feeds[currentFeed]['type']="rss1"; // 
			}
			else feeds[currentFeed]['type']="rss2"; // default type if not specified
		}
		// if status is not OK or responseXML is null
		else response=null;
		// call the method that will process the response
		processResponseRSS(response);
	}
}   

//-----------------------------------------------------//
// Data Handling Utilities                             //
//-----------------------------------------------------//

// Helper function to sort news by date descending
function compareDate(n1, n2){
	// round to compare only up to seconds level
	t1=Math.round(n1['date'].getTime()/1000);
	t2=Math.round(n2['date'].getTime()/1000);
	return t2 - t1;
}

// Helper function to sort bookmarks by name
function compareName(n1, n2){
	return n2['name']<n1['name'];
}

// Parse a standard-formatted date YYYY-MM-DDTHH:MM:SS+TMZ
function parseDate(dateToParse){
	// if dateToParse is not null
	var returnDate;
	if(!isNull(dateToParse) && dateToParse.length>5){
		// if it is in YYYY-MM-DDTHH:MM:SS format, format it
		if(dateToParse.match(/^\d\d\d\d-\d\d/)){
			// separate date and time
			var dateTime=dateToParse.split("T");
			// format date
			var dateArray=dateTime[0].split("-"); // separate year, month and day
			dateTime[0]=dateArray[2]+" "+months[dateArray[1]*1]+" "+dateArray[0]; // day monthName year
			// store the date
			dateToParse=dateTime[0];
			// format time
			if(!isNull(dateTime[1])){
				dateTime[1]=dateTime[1].replace(/\.\d+/, ""); // remove fractions of second
				dateTime[1]=dateTime[1].replace(/(\d\d?:\d\d?:\d\d?)/, "$1 "); // add a space before time zone
				dateTime[1]=dateTime[1].replace(/([\+\-]\d\d?):(\d\d?)/, "$1$2"); // remove : in time zone
				dateTime[1]=dateTime[1].replace(/(:\d+)\s*Z\b/, "$1"); // remove the Z time zone
				// store the time
				dateToParse+=" "+dateTime[1];
			}
		}
		// if it is in Day Mon MM HH:MM:SS TZ YY format, format it
		else if(dateToParse.match(/\s\d\d\d\d$/)){
			// separate components
			var dateTime=dateToParse.split(" ");
			// put them back together in the right order
			// Day, Mon MM YY HH:MM:SS TZ
			dateToParse=dateTime[0]+", "+dateTime[1]+" "+dateTime[2]+" ";
			dateToParse+=dateTime[5]+" "+dateTime[3]+" "+dateTime[4];
		}
		// parse the date into a javascript date object
		returnDate = new Date(dateToParse);
	}
	else { // if dateToParse was null, return new date
		returnDate = new Date();
	}
	
	return returnDate;
}

// clean the text so it can be spoken
function cleanText(text){
	// if text is null, return empty string
	if(isNull(text) || text.length<1) text="";
	else{ // else, format it
		// remove special quotes
		var regEx = new RegExp ('&#14[78];', 'g') ;
		text = text.replace(regEx, '') ;
	
		// remove html tags and entities
		var tmpDiv=document.getElementById("tmpDiv");
		tmpDiv.innerHTML=text;
		text=tmpDiv.innerHTML;
		if(text.indexOf("<")>-1){
			// remove <> tags
			text=text.replace(/<.*?>/g, "");
		}
	
		// remove single and double quotes
		var regExForSingleQuote = new RegExp ('\'', 'g') ;
		var regExForDoubleQuote = new RegExp ('"', 'g') ;
		text = text.replace(regExForSingleQuote, '') ;
		text = text.replace(regExForDoubleQuote, '') ;

		// lower case
		text=text.toLowerCase();
	}

	return text;
}

//-----------------------------------------------------//
// Deal With Interface                                 //
//-----------------------------------------------------//

// Update the display with the most recent news
function updateDisplayedNews(){
	var f, tmpContent, tmpHTML, tmpDate;
	var newItem, newElem;
	// unselect all items
	selectedItem=-1;
	// get todays date
	var today = new Date();
	// get the content div reference
	var contentDiv=document.getElementById("content");
	// clear the area
	contentDiv.innerHTML="";
	// if there is a new version, display a message
	if(newVersion*1 > version*1){
		newItem=document.createElement("div");
		newItem.id="newVersionMessage";
		newItem.className = "list_msg";
		newItem.innerHTML="<a href='#' class='newVersion' onclick='openURL(\""+homeURL+"\");\
			return false;'>New version available: <span id='newVersion'>"+newVersion+"</span></a>";
		contentDiv.appendChild(newItem);
	}
	// verify that it is not null
	if(isNull(newsDisplay)) newsDisplay=new Array();
	// insert the new items
	for(f=0;f<newsDisplay.length;f++){
		newItem=document.createElement("div");
		newItem.id="item_"+f;
		if(f%2!=0) newItem.className = "rss_item_even";
		else newItem.className = "rss_item_odd";
		newItem.onclick=itemClicked;
		// add the title
		if(!isNull(newsDisplay[f]['title'])){
			newElem=document.createElement("div");
			newElem.className = "rss_title";
			tmpHTML=newsDisplay[f]['title'];
			// add the source
			if(!isNull(newsDisplay[f]['source'])){
				tmpHTML+=" <span class='rss_source'>";
				tmpHTML+=newsDisplay[f]['source'];
				tmpHTML+="</span>";
			}
			// add the date
			if(!isNull(newsDisplay[f]['date'])){
				tmpDate=newsDisplay[f]['date'];
				tmpHTML+=" <span class='rss_date'>";
				// if date is not today, show it
				if(tmpDate.getDate()!=today.getDate() ||
					tmpDate.getMonth()!=today.getMonth() ||
					tmpDate.getFullYear()!=today.getFullYear()){
					// print the date
					tmpHTML+=(tmpDate.getMonth()+1)+"/"+tmpDate.getDate()+"/"+tmpDate.getFullYear();
				} else{ // else, only show hour
					if(tmpDate.getHours()<10) tmpHTML+="0";
					tmpHTML+=tmpDate.getHours()+":";
					if(tmpDate.getMinutes()<10) tmpHTML+="0";
					tmpHTML+=tmpDate.getMinutes();
				}
				tmpHTML+="</span>";
			}
			newElem.innerHTML=tmpHTML;
			newItem.appendChild(newElem);
		}
		// add the description
		if(!isNull(newsDisplay[f]['description'])){
			newElem=document.createElement("div");
			newElem.className = "rss_description";
			tmpContent=newsDisplay[f]['description'];
			// add the link to Read more...
			if(newsDisplay[f]['link'].length>1) tmpContent+=" <a href='#' id='readMore_"+f+"' onClick='javascript:openURL(\""+newsDisplay[f]['link']+"\")' class='readMore'>Read more...</a>";
			newElem.innerHTML=tmpContent;
			newItem.appendChild(newElem);
		}
		contentDiv.appendChild(newItem);
	}
	// add a blank space at the end
	newItem=document.createElement("div");
	newItem.innerHTML="&nbsp;";
	contentDiv.appendChild(newItem);
	// if there are no feeds or news, display a message
	if(feeds.length<1) showMessage("You have no bookmarks of RSS feeds.<br>Add some in Safari. <a href='#' onclick='displayHelp(); return false;'>Need help?</a>");
	else if(newsDisplay.length<1) showMessage("There are no items to display.");
	// update the count display
	document.getElementById("itemsCount").innerHTML=newsDisplay.length+" items";
}

// What to do when mouse is over the play/pause button
function mouseOverButton(){
	// get a reference to the button image
	type=determineButtonType();
	buttonImg=document.getElementById("playButtonImg");
	// change image to mouse over version
	buttonImg.src="Images/"+type+"_mo.png";
}

// What to do when mouse goes out of the play/pause button
function mouseOutButton(){
	// get a reference to the button image
	type=determineButtonType();
	buttonImg=document.getElementById("playButtonImg");
	// change image to normal version
	buttonImg.src="Images/"+type+".png";
}

// What to do when the mouse is pressed over the play/pause button
function mouseDownButton(){
	// get a reference to the button image
	type=determineButtonType();
	buttonImg=document.getElementById("playButtonImg");
	// change image to mouse down version
	buttonImg.src="Images/"+type+"_md.png";
}

// What to do when the mouse is released over the play/pause button
function mouseUpButton(){
	// if it is paused, play
	if(!currentlyPlaying){
		startSpeakItems();
	} else{ // if it is playing, pause
		pauseSpeakItems();
	}
}

// Determine if the button is play or pause
function determineButtonType(){
	if(currentlyPlaying) return "pause";
	else return "play";
}


// Open a URL in the browser
function openURL(url){
	if(window.widget) widget.openURL(url);
	else document.location.href=url;
}

// What to do when the user clicks the item
function itemClicked(){
	// pause the speaking
	pauseSpeakItems();
	// select the item
	index=this.id.substr(this.id.indexOf("_")+1, this.id.length);
	if(selectedItem==index) unselectItem(index);
	else selectItem(index);
}


// Select an item
function selectItem(index){
	itemToSelect=document.getElementById("item_"+index);
	if(!isNull(itemToSelect)){
		// unselect all others
		for(f=0;f<newsDisplay.length;f++){
			if(f%2!=0) document.getElementById("item_"+f).className="rss_item_even";
			else document.getElementById("item_"+f).className="rss_item_odd";
		}
		// select this
		itemToSelect.className="rss_item_selected";
		selectedItem=index;
		// focus its link
		var link=document.getElementById("readMore_"+selectedItem);
		// commented out because it does not scroll to the desired item
		// if(!isNull(link)) link.focus();
	}
}

// Unselect an item
function unselectItem(index){
	itemToUnselect=document.getElementById("item_"+index);
	if(!isNull(itemToUnselect)){
		// unselect this
		if(index%2!=0) itemToUnselect.className="rss_item_even";
		else itemToUnselect.className="rss_item_odd"; 
		selectedItem=-1;
	}
}


// Search
function search(terms){
	newsDisplay=new Array();
	// pause the speaking
	pauseSpeakItems();
	// if there are no terms, display all
	if(terms.length<2) newsDisplay=newsMaster;
	else{ // filter out
		// escape special regexp characters
		terms=terms.replace(/([\$\/\.\+\*\\])/ig, "\\$1");
		// create the regula expression
		var re = new RegExp(terms, "i")
		var reParen = new RegExp("("+terms+")", "ig")
		// filter the news that contain the terms
		for(f=0;f<newsMaster.length;f++){
			// if it matches the terms
			if(newsMaster[f]['title'].match(re) || newsMaster[f]['description'].match(re)){
				// copy it into the display array
				newsDisplay[newsDisplay.length] = new Array();
				for(var i in newsMaster[f]){
					// hilight found terms
					if(i=="description" || i=="title"){
						newsDisplay[newsDisplay.length-1][i] = 
							newsMaster[f][i].replace(reParen, "<span class='hilighted'>$1</span>");
					}
					else newsDisplay[newsDisplay.length-1][i]=newsMaster[f][i];
				}
			}
		}
	}
	// update the display
	updateDisplayedNews();
}

// Show a message in the main content area
function showMessage(msg){
	// get the content div reference and display the message
	contentDiv=document.getElementById("content");
	contentDiv.innerHTML="<p class='message'>"+msg+"</p>";
}

// Show online help to add RSS bookmarks
function displayHelp(){
	// help message
	var msg="<p class='messageHelp'>Vocero uses Safari bookmarks of RSS feeds as sources for the news it displays.</p>";
	msg+="<p class='messageHelp'>To add bookmarks, go to Safari and enter a news website. \
		If the blue RSS button appears in the address bar, click it. Then choose the \"Bookmarks -&gt; Add Bookmark...\"\
		option in the menu.</p>";
	msg+="<p class='messageHelp'>Now you should see your RSS bookmarks in the back side (preferences) of Vocero.\
		Select the ones you want to use as sources of the displayed news.</p>";
	// get the content div reference and display the message
	contentDiv=document.getElementById("content");
	contentDiv.innerHTML=msg;
}

//-----------------------------------------------------//
// Deal With Speaking                                  //
//-----------------------------------------------------//

// start to speak all the news (items)
function startSpeakItems(){
	currentlyPlaying=true;
	// change the button to pause
	buttonImg=document.getElementById("playButtonImg");
	buttonImg.src="Images/pause.png";
	// if there are no items pause speaking
	if(newsDisplay.length<1) pauseSpeakItems();
	else{
		// if no item is selected, select the first one
		if(selectedItem<0 || selectedItem>=newsDisplay.length) selectItem(0);
		else selectItem(selectedItem);
		// speak the items header
		speakItemHeader();
	}
}

// pause the speaking
function pauseSpeakItems(){
	// change the button to play
	buttonImg=document.getElementById("playButtonImg");
	buttonImg.src="Images/play.png";
	// stop speaking
	currentlyPlaying=false;
	if( currentlyBeingSpoken != null) {
		currentlyBeingSpoken.cancel();
	}
}

// speak an items header
function speakItemHeader(){
	// validate that there is a valid voice
	if(prefs['headerVoice'].length<1) return false;
	// prepare the text to be spoken
	var textToSpeak=cleanText(newsDisplay[selectedItem]['title']);
	// speak the text
	if(window.widget) {
		// if it is speaking, stop it
		if( currentlyBeingSpoken != null) {
			currentlyBeingSpoken.cancel();
		}
		currentlyBeingSpoken = widget.system("/usr/bin/osascript -e 'say \"" + textToSpeak + "\" using \"" + prefs['headerVoice'] + "\"'" , speakItemText);
	}
}

// speak an items text
function speakItemText(){
	// if there is no selected voice, go to the next item
	if(prefs['textVoice'].length<1) speakNextItem();
	else{
		// prepare the text to be spoken
		var textToSpeak=cleanText(newsDisplay[selectedItem]['description']);
		// speak the text
		if(window.widget) {
			// if it is speaking, stop it
			if( currentlyBeingSpoken != null) {
				currentlyBeingSpoken.cancel();
			}
			currentlyBeingSpoken = widget.system("/usr/bin/osascript -e 'say \"" + textToSpeak + "\" using \"" + prefs['textVoice'] + "\"'" , speakNextItem);
		}
	}
}

// go to the next item to be spoken
function speakNextItem(){
	// if it was the last one, finish speaking
	if(selectedItem>=newsDisplay.length-1){
		unselectItem(selectedItem);
		pauseSpeakItems();
	} else{ // if it was not the last one
		// select next item
		selectedItem++;
		selectItem(selectedItem);
		startSpeakItems();	
	}
}

// indicate theres nothing being spoken
function doneSpeaking(){
	currentlyBeingSpoken = null;
}


//-----------------------------------------------------//
// Deal With Preferences                               //
//-----------------------------------------------------//

// Check if there is a new version
function checkNewVersion(){
	// send a request to the server
	reqCheckVersion = new XMLHttpRequest();
	reqCheckVersion.onreadystatechange = processCheckNewVersion;
	// add a random parameter to the url to eliminate cache issues
	if(checkVersionURL.indexOf("?")!=-1) checkVersionURL += "&"+Math.random();
	else checkVersionURL += "?"+Math.random();
	// send the request to the server
	reqCheckVersion.open("GET", checkVersionURL, true);
	reqCheckVersion.send(null);
}

// Process a check new version request
function processCheckNewVersion(){
	// verify that the status is "COMPLETE"
	if (reqCheckVersion.readyState == 4) {
		// verify that the status response is "OK"
		if (reqCheckVersion.status>=200 && reqCheckVersion.status<300 && !isNull(reqCheckVersion.responseXML)) { // if status is 2xx
			// create XML object
			response = reqCheckVersion.responseXML.documentElement;
			// extract new version number
			tmpValue=response.getElementsByTagName("versionNumber")[0];
			if(!isNull(tmpValue)) newVersion=tmpValue.firstChild.data;
			else newVersion=0;
			// if it is greater than current, display message
			if(newVersion*1 > version*1){
				document.getElementById("newVersion").innerHTML=newVersion;
				document.getElementById("newVersionDiv").style.display="block";
			} else{
				document.getElementById("newVersionDiv").style.display="none";
			}
		}
	}	
}

// What to do when widget is removed from dashboard
function widgetRemoved(){
	// pause speaking items
	pauseSpeakItems();
	// remove preferences
	if(window.widget){
		// commented because it was preferred not to
	 	//widget.setPreferenceForKey(null, "");
	 }
}

// Load user preferences into prefs global array
function readPreferences(){
	var tmpPref;
	// voice to read the headers
	tmpPref=getPreference("headerVoice");
	if(!isNull(tmpPref) && tmpPref.length>0) prefs['headerVoice']=tmpPref;
	// voice to read the texts
	tmpPref=getPreference("textVoice");
	if(!isNull(tmpPref) && tmpPref.length>0) prefs['textVoice']=tmpPref;
	// time to refresh
	tmpPref=getPreference("refreshTime");
	if(!isNull(tmpPref) && tmpPref>0) prefs['refreshTime']=tmpPref*1;
	
}

// Load RSS bookmars as feeds
function loadBookmarks(){
	var command, outputString, newItem, status, f, bookmarkFiles;
	feeds=new Array(); // initialize feeds array
	if(window.widget){
		// read bookmarks from perl script
		command="/usr/bin/perl Scripts/loadBookmarks.pl";
		outputString = widget.system(command, null).outputString;
		if(!isNull(outputString) && outputString.length>5){
			bookmarkFiles=outputString.split("\n"); // split the results string by rows
			if(!isNull(bookmarkFiles)){
				// first line says how many are there and if there are more
				bookmarksFound=bookmarkFiles[0];
				// process each bookmark file
				for(f=1;f<bookmarkFiles.length;f++){
					if(bookmarkFiles[f].length>1){
						var tmpBookmark=new Array();
						var tmpArray=new Array();
						tmpArray=bookmarkFiles[f].split("|=|");
						// get the name of the bookmark
						tmpBookmark['name'] = tmpArray[0];
						// get the url
						tmpBookmark['url'] = tmpArray[1];
						// store it in the array
						feeds[feeds.length]=tmpBookmark;
					} // end if bookmarkfiles.length
				} // end for f
			} // end if !isNull
			// if it is the first run, select the first one
			if(firstRun){
				if(!isNull(feeds[0])) setPreference(feeds[0]['url'], "1");
			}
		} // end if outputstring length > 5
	} // end if widget
}

// Refresh the display of preferences
function refreshPreferences(forceLoadBookmarks){
	var f, tmpHTML;
	var bookmarksDiv=document.getElementById("pref_sourcesDiv");
	// refresh the selected voices
	var headerVoice=document.getElementById("selectHeaderVoice");
	var textVoice=document.getElementById("selectTextVoice");
	for(f=0;f<headerVoice.options.length;f++){
		if(headerVoice.options[f].value==prefs['headerVoice'])
			headerVoice.options[f].selected=true;
		if(textVoice.options[f].value==prefs['textVoice'])
			textVoice.options[f].selected=true;
	}
	// refresh the display of time to refresh
	document.getElementById("inputRefreshTime").value = prefs['refreshTime'];
	// display message of loading bookmarks and load them
	bookmarksDiv.innerHTML="<p class='messageBack'>Loading Safari RSS Bookmarks...</p>";
	// if there are no bookmarks, read them again
	if(forceLoadBookmarks || isNull(feeds) || feeds.length<1) loadBookmarks();
	bookmarksDiv.innerHTML="";
	// update displayed bookmarks
	if(!isNull(feeds) && feeds.length>0){
		// if there are more than allowed, show message
		if(bookmarksFound>bookmarksLimit){
			newItem=document.createElement("div");
			newItem.className = "source_msg";
			newItem.innerHTML="You have "+bookmarksFound+" bookmarks and the max.\
				allowed is "+bookmarksLimit+". Some will not be displayed.";
			bookmarksDiv.appendChild(newItem);
		}
		// insert each bookmark
		for(f=0; f<feeds.length; f++){
			newItem=document.createElement("div");
			if(f%2==0) newItem.className = "source_odd";
			else newItem.className = "source_even";
			tmpHTML="<input type=\"checkbox\" name=\"source_"+f+"\" value=\""+feeds[f]['url']+"\"";
			tmpHTML+=" onClick='changedBookmark(this);' ";
			if(isFeedSelected(feeds[f]['url'])) tmpHTML+=" checked";
			tmpHTML+="> "+feeds[f]['name'];
			newItem.innerHTML=tmpHTML;
			bookmarksDiv.appendChild(newItem);
		}
	} else{ // if there are no bookmarks
		bookmarksDiv.innerHTML="<p class='messageBack'>No Safari RSS Bookmarks found.</p>";
	}
}

// Determine if a feed is selected
function isFeedSelected(url){
	// get its status
	var status=getPreference(url);
	if(!isNull(status) && status=="1") return true
	else return false;
}

// Refresh the list of voices
function refreshVoices(){
	// refresh list of voices
	var headerVoice=document.getElementById("selectHeaderVoice");
	var textVoice=document.getElementById("selectTextVoice");
	for(f=0;f<voices.length;f++){
		if(f>0) headerVoice.options[f-1] = new Option(voices[f], voices[f], 0, 0);
		textVoice.options[f] = new Option(voices[f], voices[f]!="--None--"?voices[f]:"", 0, 0);
	}
}

// What to do when a bookmark checkbox changed
function changedBookmark(chk){
	// if it is checked, save it
	if(chk.checked){
		setPreference(chk.value, "1");
	} else { // otherwise, delete it
		setPreference(chk.value, null);
	}
	// indicate that it is necesary to refresh when flipping back to front
	needsRefresh=true;
}

// What to do when a voice selector changed
function changedVoice(type, chosenVoice){
	var command;
	// say a sample
	if(window.widget && chosenVoice.length>0) {
		if(currentlyBeingSpoken != null) {
			currentlyBeingSpoken.cancel();
			doneSpeaking();
		}
		// determine what to say and say it
		if(type=="headerVoice") what="headers";
		else what="text";
		command="/usr/bin/osascript -e 'say \"Hi, I will read the "+what+"\" using \""+chosenVoice+"\"'";
		currentlyBeingSpoken = widget.system(command, doneSpeaking);
	}
	// save the value
	setPreference(type, chosenVoice);
}

// Save a preference value
function setPreference(key, pref){
	var validated=false;
	if(window.widget){
		// validate
		if(key=="refreshTime"){
			pref=pref*1;
			if(pref>0) validated=true;
		}
		else if(key=="headerVoice" || key=="textVoice"){
			for(f=0;f<voices.length;f++){
				if(pref==voices[f] || pref==""){
					validated=true;
					break;
				}
			}
		}
		else validated=true;
		// save the value
		if(validated){
			// in memory
			prefs[key]=pref;
			// permanent
	 		widget.setPreferenceForKey(pref, key);
	 	}
	}
}

// Retrieve a preference value
function getPreference(key){
	if(window.widget){
 		return widget.preferenceForKey(key);
	}
	else return "";
}

